package ru.ulstu.services;

import org.springframework.stereotype.Service;
import ru.ulstu.models.ModelingResult;
import ru.ulstu.models.TimeSeries;
import ru.ulstu.tsMethods.TimeSeriesMethod;
import ru.ulstu.tsMethods.exponential.param.TimeSeriesMethodParam;
import ru.ulstu.tsMethods.exponential.param.TimeSeriesMethodParamValue;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

@Service
public class MethodParamBruteForce {
    private final List<TimeSeriesMethod> methods;
    private ExecutorService executors = Executors.newFixedThreadPool(10);

    public MethodParamBruteForce(List<TimeSeriesMethod> methods) {
        this.methods = methods;
    }

    public TimeSeries getForecast(TimeSeries timeSeries) {

    }

    public TimeSeries getForecastWithOptimalLength(TimeSeries timeSeries) {

    }

    public TimeSeries getSmoothedTimeSeries(TimeSeries timeSeries) {
        List<Future<ModelingResult>> results = new ArrayList<>();
        for (TimeSeriesMethod method : methods) {
            List<TimeSeriesMethodParam> availableParams = method.getAvailableParams();
            Map<TimeSeriesMethodParam, List<Number>> paramsAvailableValues = new HashMap<>();
            for (TimeSeriesMethodParam param : availableParams) {
                paramsAvailableValues.put(param, param.getAvailableValues());
            }
            List<TimeSeriesMethodParamValue> paramValues = new ArrayList<>();
            results.add(executors.submit(() -> {
                return new ModelingResult(method.getModel(),
                        method.createFor(timeSeries)
                                .setAvailableParams(paramValues)
                                .getModel());
            }));
        }
        TimeSeries result = null;
        try {
            result = results.get(0).get();
        } catch (Exception e) {
            e.printStackTrace();
        }

        return result;
    }
}
